'''Configurable constants are stored in this file. You must change
BASE_URL andBASE_PATH to your website's URL and the full
path to your home directory.
'''

# constants holds values that shouldn't be changed
from constants import *
import pickle

######################### CHANGE ############################
# This script tries to load the base_url and base_path from the
# config file, if it exists.  You can either change the constants
# here (make sure to delete the "config" file if you do change
# them here so a new one is created), or modify these constants
# through the web interface (TODO not implemented yet).
try:
    file = open(CONFIG_FILE)
    dict = pickle.load(file)
    file.close()
    
    # BASE_URL is given to the users when a survey is created,
    # and is used for forwarding.
    # It should be the full URL of your website (up to one
    # directory above the survey tool).
    BASE_URL        = dict['base_url']
    
    # BASE_PATH is used in the creation of .htaccess and .htpasswd
    # files.  It should be the full path to the directory
    # just above the survey tool (e.g. /usr/home/public_html/)
    BASE_PATH       = dict['base_path']
    
    # The mailserver is used to send out reset passwords to survey
    # creators and to send out survey invitations.
    MAILSERVER          = dict['mailserver']
    
    # Optionally, you can enter a default address
    # that mail will be sent from if another address is not entered.
    DEFAULT_REPLY_EMAIL = dict['default_email']

    # Set to 1 if you want new users to be able to make accounts.
    ALLOW_NEWUSERS      = dict['newusers']
    
    # Set to 1 if you want new users to be able to reset their
    # password.
    SHOW_FORGOTPASSWORD = dict['forgotpassword']
    
    # Set to 1 if you want new users to be able to do account management
    # tasks such as changing their password or email address.
    SHOW_ACCOUNTMGMT    = dict['accountmgmt']
    
    # Default_openness determines whether a newly-created survey is 
    # by default OPEN (which allows responses) or CLOSED (which does
    # not allow responses).  Openness for a particular survey
    # survey can be changed from the main editing menu.
    DEFAULT_OPENNESS    = dict['openness']
    
    # This affects the open/close form only.
    # The open date is by default set for the current date,
    # and the closing date is set for this many days from the
    # current date.
    DEFAULT_OPEN_DAYS   = dict['open_days']
    
    # This is the text that is shown when someone tries to access
    # a closed survey.
    CLOSED_TEXT         = dict['closed_text']

except:
    BASE_URL            = 'http://www.ocf.berkeley.edu/~ucbd/'
    BASE_PATH           = '/services/http/users/u/ucbd/'
    MAILSERVER          = 'mail.ocf.berkeley.edu'
    DEFAULT_REPLY_EMAIL = 'no-reply@mailinator.com'
    ALLOW_NEWUSERS      = 1
    SHOW_FORGOTPASSWORD = 0
    SHOW_ACCOUNTMGMT    = 0
    DEFAULT_OPENNESS    = CLOSED
    DEFAULT_OPEN_DAYS   = 7
    CLOSED_TEXT         = 'This survey is currently closed.'
    dict                = {'base_url': BASE_URL,
                           'base_path': BASE_PATH,
                           'mailserver': MAILSERVER,
                           'default_email': DEFAULT_REPLY_EMAIL,
                           'newusers': ALLOW_NEWUSERS,
                           'forgotpassword': SHOW_FORGOTPASSWORD,
                           'accountmgmt': SHOW_ACCOUNTMGMT,
                           'open_days': DEFAULT_OPEN_DAYS,
                           'openness': DEFAULT_OPENNESS,
                           'closed_text': CLOSED_TEXT}
    file = open(CONFIG_FILE, 'w')
    pickle.dump(dict, file)
    file.close()

# This is the name of the directory that holds the survey scripts.
# Unless you change the name of the survey directory after you
# unzip it, you shouldn't need to change this.
SURVEY_DIR          = 'survey/'
BASE_URL += SURVEY_DIR
BASE_PATH += SURVEY_DIR

# Htaccess and htpasswd are Apache utilities that enable easy
# password protection of directories.  This is used to provide
# extra protection for the data directories (they are also not
# readable to anyone but the "owner" of the file by default).
# Check the location of your htpasswd by typing 'which htpasswd'
# at a command prompt.  If it's different than what is printed
# below, replace what is below.
HTPASSWD_LOCATION   = '/opt/local/bin/htpasswd'

# These are the htpasswd options for the creation of an .htpasswd
# file.  If you have a different version of htpasswd, you'll want
# to change this.
HTPASSWD_CREATE     = ' -cb '
# This one is currently not used, but will be if more than one owner
# is allowed per survey in a future version
HTPASSWD_APPEND     = ' -b '

# This should be set to the default encryption used by htaccess
# on your system.  It's usually crypt, though md5 encryption is
# stronger.
DEFAULT_ENCRYPTION = CRYPT

# SURVEY_PATH is the path of the survey data RELATIVE to the
# survey directory. If you don't want your survey data stored
# in the survey directory, you must change this.
SURVEY_PATH         = '.'

# You'll need to change these .cgi extensions to .py extensions
# if you put the survey tool in a cgi-bin directory and can't
# run files with .cgi extensions.  Some server configurations
# require this.
PROCESS_SURVEY_CGI  = 'index.cgi'
INPUT_SURVEY_CGI    = 'edit.cgi'
LOGIN_CGI           = 'login.cgi'
LOGOUT_CGI          = 'logout.cgi'
NEWUSER_CGI         = 'newuser.cgi'
CHANGE_USERINFO_CGI = 'change_userinfo.cgi'

# You shouldn't need to change anything below this, but
# you can if you want.

DEBUGGING           = 1
debugging           = DEBUGGING

# These are the options that show up on the main editing page.
EDIT_OPTIONS        = [[INPUT_SURVEY_CGI, \
                        '<b>Create new survey</b>\n'], \
                       [EDIT_SURVEY, \
                        '<b>Modify existing survey</b>'], \
                       [PREVIEW_SURVEY_CGI, \
                        '<b>Preview survey</b> all on one page'], \
                       [TESTING, \
                        '<b>Test survey</b> without saving your responses'], \
                       [PROCESS_SURVEY_CGI, \
                        '<b>Take survey</b> and save responses (only when survey is open)'], \
                       # TODO [LOCK_SURVEY, \
                       # '<b>Lock survey</b> against major edits, for data integrity.'], \
                       [OPEN_CLOSE_SURVEY, \
                        '<b>Open or close survey</b> or set opening/closing dates'], \
                       #[ENTER_PARTICIPANTS, \
                       # '<b>Generate usernames and passwords</b> ' + \
                       # 'for surveys with a closed pool of participants, and ' + \
                       # 'optionally email participants their login information'], \
                       [DELETE_SURVEY, \
                        '<b>Delete survey</b> and all responses <i>permanently</i>'], \
                       # enter more survey editing functions here
                       ]

# the directory where survey responses are stored
DATA_DIR            = 'data'

# the files where survey information is stored
SURVEY_FILE         = 'surveycontents'
SURVEY_MODS         = 'surveymodifiers'

# size of the text input area
TEXT_INPUT_ROWS     = 30
TEXT_INPUT_COLS     = 80

# survey formatting and field names
OTHER_FIELD_NAME      = '-other'
OTHER_TEXTINPUT_SIZE  = 10
OTHER_TEXTAREA_HEIGHT = 2
OTHER_TEXTAREA_WIDTH  = 10

# codewords can be used in survey creation by power users.
# For example, %PAGE% is replaced by the current page number
# of the survey.
CODEWORDS           = {'%SURVEYID%': 'self.get(SURVEY_ID)', \
                       '%USERID%': 'USER_ID', \
                       '%PAGE%': 'str(PAGENUM)', \
                      }

# The key is replaced by its value to make the string CSV-friendly.
csv_dict            = {'\r': '', \
                       '\n': '\\n', \
                       '"': "''", \
                      }

# The key is replaced by its value to make the string HTML form-friendly.
html_dict_keys      = ['\r', '\n', '<', '>', '"']
html_dict           = {html_dict_keys[0]: '', \
                       html_dict_keys[1]: '<br>', \
                       html_dict_keys[2]: '&#60;', \
                       html_dict_keys[3]: '&#62;', \
                       html_dict_keys[4]: '&#34;', \
                      }

# The key is replaced by its value to format the string for HTML.
htmlize_dict_keys   = ['\r', '\n\n', '\n']
htmlize_dict        = {htmlize_dict_keys[0]: '', \
                       htmlize_dict_keys[1]: '<p>', \
                       htmlize_dict_keys[2]: '<br>', \
                      }

# The key is replaced by its value to make the string HTML form-friendly.
punctuation_dict    = {' ': '_', \
                      #'_': '_', \
                      #'-': '-', \
                      '.': '', \
                      ',': '', \
                      '<': '', \
                      '>': '', \
                      '?': '', \
                      '/': '', \
                      ';': '', \
                      ':': '', \
                      '"': '', \
                      "'": '', \
                      '!': '', \
                      '@': '', \
                      '#': '', \
                      '$': '', \
                      '%': '', \
                      '^': '', \
                      '&': '', \
                      '*': '', \
                      '(': '', \
                      ')': '', \
                      '=': '', \
                      '+': '', \
                      '[': '', \
                      ']': '', \
                      '{': '', \
                      '}': '', \
                      '\\': '', \
                      '|': '', \
                      '~': '', \
                      '`': '', \
                      }

# used in authentication
DEFAULT_DEST         = INPUT_SURVEY_CGI
SPECIAL_DESTINATIONS = [PREVIEW_SURVEY_CGI, SAVE_SURVEY_CGI]

# this is the formatting information shown below
# the survey editing box.

QUESTION_NAME       = 'question_name'
HELP_HEIGHT         = '400'
HELP_WIDTH          = '300'

EXAMPLE_JAVASCRIPT  = '''(<a href="javascript:void(0)" onclick="window.open('help.cgi?%s=%s', '', 'height='''+HELP_HEIGHT+', width='+HELP_WIDTH+''', resizable=1, scrollbars=yes')">example</a>)'''

question_formatting = [
    'Vertical radio buttons ' + (EXAMPLE_JAVASCRIPT % (QUESTION_NAME, V_RADIO)) + \
        ''': enter each option on a new line below the question text,
        with three spaces and an asterisk (*) before it.''', \
    'Vertical checkboxes ' + (EXAMPLE_JAVASCRIPT % (QUESTION_NAME, V_CHECKBOX)) + \
        ''': enter each option on a new line below the question text, with three spaces an a pound sign (#) before it''', \
    'Horizontal (Likert) radio buttons ' + (EXAMPLE_JAVASCRIPT % (QUESTION_NAME, H_RADIO)) + \
        ''': enter the scale, separated by tabs (\\t) and with a tab at the beginning,
        on the top line. Enter the items to
        be scaled, each on its own line, with a tab and an asterisk (*) after each.''', \
    'Horizontal checkboxes ' + (EXAMPLE_JAVASCRIPT % (QUESTION_NAME, H_CHECKBOX)) + \
        ''': enter the scale, separated by tabs (\\t) and with a tab at the beginning,
        on the top line. Enter the items to
        be scaled, each on its own line, with a tab and a pound sign (#) after each.''', \
    'Drop-down selection boxes ' + (EXAMPLE_JAVASCRIPT % (QUESTION_NAME, SELECT)) + \
        ''': enter each option on a new line below the question text,
        with three spaces and a hyphen (-) before it.''', \
    'Text box ' + (EXAMPLE_JAVASCRIPT % (QUESTION_NAME, TEXTINPUT)) + \
        ''': on a new line below the question, input at least 3 underscores (_).
        The number of underscores determines the size of the text box.''', \
    'Password box ' + (EXAMPLE_JAVASCRIPT % (QUESTION_NAME, PASSWORD)) + \
        ''': on a new line below the question, input at least 3 asterisks (*).
        The number of asterisks determines the size of the password box.''', \
    'Multi-line text field ' + (EXAMPLE_JAVASCRIPT % (QUESTION_NAME, TEXTAREA)) + \
        ''': enter |-------| on the line after the question
        (where the number of hyphens represents the width of the box, min. 3),
        and || on additional lines up to the desired height of the box.''', \
    'Page titles ' + (EXAMPLE_JAVASCRIPT % (QUESTION_NAME, PAGETITLE)) + \
        ''': add [pagetitle] to the beginning of the title line.''', \
    'Page breaks ' + (EXAMPLE_JAVASCRIPT % (QUESTION_NAME, PAGEBREAK)) + \
        ''': --PAGEBREAK-- on a line by itself (more dashes can be added if you want).''', \
    'Reset buttons ' + (EXAMPLE_JAVASCRIPT % (QUESTION_NAME, RESET)) + \
        ''': [reset] on a line by itself.''', \
    ]

survey_formatting = '''
<b>Formatting tips:</b>
<p>
Questions are separated by <i>two</i> newlines.  You can
create the following types of questions:
<ul>
  <li> ''' + '\n  <li> '.join(question_formatting) + '''
</ul>

'''

question_examples = {
    V_RADIO: '''This is the text for a radio question.  Survey-takers can select just <i>one</i> of the following options.  The question text must be separated from the options by just one line.  The radio question options must have three spaces followed by a asterisk character (*).  Putting 'other' as the last option will render as 'other' plus a text box.
   * option 1
   * option 2
   * option 3
   * other''', \
    V_CHECKBOX: '''This is an example of a checkbox question, very similar to the radio question, except that survey-takers can select as many of the options as they want.  Denote options by putting the hash or pound character (#) after three spaces.  Putting 'other' as the last option will render as 'other' plus a text box.
   # option 1
   # option 2
   # option 3
   # option 4
   # option 5
   # other''', \
    SELECT: '''This is an example of a dropdown selection box.  Though its format is similar to the radio and checkbox questions above, it renders much differently, and is especially well-suited to long lists (such as state or country names).  Survey-takers click on the drop-down arrow and use the mouse or keyboard to select one of the options from the list.  The formatting for this question is similar to radio and checkbox questions, except that a dash character (-) is used at the beginning of each option.  The first option is the default.  To have a blank option as the first one, just add the three spaces and dash, as shown here.
   - 
   - option 1
   - option 2
   - option 3
   - option 4''', \
    H_RADIO: '''This horizontal radio (or likert scale) question allows survey-takers to select one option in each row, and is often used for a set of questions that all use the same scale.  The likert scale options (across the top) must start with a tab, and the options below must be followed by <i>at least one</i> tab and asterisk character (*).  Putting more than one in is optional.
\t\t1 (None)\t2\t3\t4\t5 (Very)\tDon't know
option 1\t*
option 2\t*
option 3\t*
option 4\t*
other\t\t*''', \
    H_CHECKBOX: '''The horizontal checkbox question is like the horizontal radio question above, but is less commonly used.  Survey-takers can select as many options as they want in each row.  Formatting is the same as for the horizontal radio question, except that asterisks are replaced with hash characters (#).
\t\tno\tmaybe\tyes
option 1\t#
option 2\t#
option 3\t#
option 4\t#
other\t\t#''', \
    TEXTINPUT: '''This is an example of a one-line text input field.  The number of underscores determines the width of the field (in characters, min. 3).  (Note that not all browsers render this correctly.)  This one is 15 characters wide.
_______________''', \
    TEXTAREA: '''This is an example of a multi-line text input field.  The number of dashes in the top line determines the width (in characters, min. 3), and the number of lines past the top line determines the height (in lines, min. 1). (Note that not all browsers render these dimensions correctly.) This textbox will be 10 characters wide and 4 lines high.  Putting the spaces in lines after the first one isn't necessary, but makes it more readable for you.
|----------|
|          |
|          |
|          |
|----------|

This is another example of a multi-line text input field, with a different size.  Note that the lines after the first one are not expanded; this doesn't matter.  This field will be 25 characters across and 2 lines high.
|-------------------------|
||''', \
    PASSWORD: '''This is an example of a password field.  The number of asterisks determines the width of the field (in characters, min. 3). (Note that not all browsers render this correctly.) This one is 5 characters wide.  Text typed into this field will appear as asterisks.
*****''', \
    PAGEBREAK: '''-------------------PAGEBREAK---------------------

--pagebreak--''', \
    PAGETITLE: '''[pagetitle] This is a page title.''', \
    TEXT: '''This is general text, not attached to any question.  General text is not numbered.  Linebreaks are converted to HTML linebreaks, but all other HTML formatting must be entered manually.  Survey question numbers are added automatically (but general text like this is not numbered, nor are titles).<p>All elements in the survey (questions, general text, titles, pagebreaks, etc.) are separated by a blank line.  One particular element should never have a blank line in it.  To add a blank line, use the HTML paragraph tag (&lt;p&gt;).''', \
    RESET: '''[reset]''', \
    }

# this is the data for the included Sample Survey.
sample_survey_title = 'Sample Survey'
test_text           = '''[pagetitle] This is a page title.

A survey usually begins with an explanation of the purpose of the survey, the time it should take the survey-taker to complete, and any informed consent information required.

The next element is a pagebreak, which you can use to divide many questions across multiple pages.  If your survey-takers back up to change responses, their latest responses will be saved.  I've included several formats for pagebreaks throughout the survey.  Pagebreaks must have two dashes on either side of the word 'pagebreak' with no spaces between.

-------------------PAGEBREAK---------------------

[pagetitle] Page 1

This is general text, not attached to any question.  General text is not numbered.  Linebreaks are converted to HTML linebreaks, but all other HTML formatting must be entered manually.  Survey question numbers are added automatically (but general text like this is not numbered, nor are titles).<p>All elements in the survey (questions, general text, titles, pagebreaks, etc.) are separated by a blank line.  One particular element should never have a blank line in it.  To add a blank line, use the HTML paragraph tag (&lt;p&gt;).

This is the text for a radio question.  Survey-takers can select just <i>one</i> of the following options.  The question text must be separated from the options by just one line.  The radio question options must have three spaces followed by a asterisk character (*).  Putting 'other' as the last option will render as 'other' plus a text box.
   * option 1
   * option 2
   * option 3
   * other

This is an example of a checkbox question, very similar to the radio question, except that survey-takers can select as many of the options as they want.  Denote options by putting the hash or pound character (#) after three spaces.  Putting 'other' as the last option will render as 'other' plus a text box.
   # option 1
   # option 2
   # option 3
   # option 4
   # option 5
   # other

This is an example of a dropdown selection box.  Though its format is similar to the radio and checkbox questions above, it renders much differently, and is especially well-suited to long lists (such as state or country names).  Survey-takers click on the drop-down arrow and use the mouse or keyboard to select one of the options from the list.  The formatting for this question is similar to radio and checkbox questions, except that a dash character (-) is used at the beginning of each option.  The first option is the default.  To have a blank option as the first one, just add the three spaces and dash, as shown here.
   - 
   - option 1
   - option 2
   - option 3
   - option 4

--PAGEBREAK--

[pagetitle] Page 2

This horizontal radio (or likert scale) question allows survey-takers to select one option in each row, and is often used for a set of questions that all use the same scale.  The likert scale options (across the top) must start with a tab, and the options below must be followed by <i>at least one</i> tab and asterisk character (*).  Putting more than one in is optional.
\t\t1 (None)\t2\t3\t4\t5 (Very)\tDon't know
option 1\t*
option 2\t*
option 3\t*
option 4\t*
other\t\t*

The horizontal checkbox question is like the horizontal radio question above, but is less commonly used.  Survey-takers can select as many options as they want in each row.  Formatting is the same as for the horizontal radio question, except that asterisks are replaced with hash characters (#).
\t\tno\tmaybe\tyes
option 1\t#
option 2\t#
option 3\t#
option 4\t#
other\t\t#

This is some more general text not attached to a question.  Text like this can be used to give instructions.  The next element is a reset button, which will be displayed next to the 'Continue' button automatically added to the end of every page.

[reset]

-------------------------PAGEBREAK-------------------

[PAGETITLE] Page 3

This is an example of a one-line text input field.  The number of underscores determines the width of the field (in characters, min. 3).  (Note that not all browsers render this correctly.)  This one is 15 characters wide.
_______________

This is an example of a password field.  The number of asterisks determines the width of the field (in characters, min. 3). (Note that not all browsers render this correctly.) This one is 5 characters wide.  Text typed into this field will appear as asterisks.
*****

This is an example of a multi-line text input field.  The number of dashes in the top line determines the width (in characters, min. 3), and the number of lines past the top line determines the height (in lines, min. 1). (Note that not all browsers render these dimensions correctly.) This textbox will be 10 characters wide and 4 lines high.  Putting the spaces in lines after the first one isn't necessary, but makes it more readable for you.
|----------|
|          |
|          |
|          |
|----------|

This is another example of a multi-line text input field, with a different size.  Note that the lines after the first one are not expanded; this doesn't matter.  This field will be 25 characters across and 2 lines high.
|-------------------------|
||

-------pagebreak----------

[pagetitle] Thank-you page

The last page of the survey usually has a thank you and a link to the main research page, if there is one.  At the end of the survey, you can optionally add a pagebreak.
'''

# currently not used; might be in the future ...
test_dict           = {'survey_id': 'testsurvey',
                       'title': 'Test Survey',
                       
                       'q001-type': 'pagetitle', 
                       'q001-text': 'This is a sample survey with sample questions.',
                       
                       'q002-type': 'vertical-checkbox',
                       'q002-text': 'Pick a color.',
                       'q002-option01': 'red',
                       'q002-option02': 'blue',
                       'q002-option03': 'green',
                       'q002-option04': 'yellow',
                       'q002-option05': 'other',
          
                       'q003-type': 'pagebreak',
                       
                       'q004-type': 'horizontal-radio',
                       'q004-text': 'Rate each:',
                       'q004-option01': 'fuzzy things',
                       'q004-option02': 'warm things',
                       'q004-option03': 'warm and fuzzy things',
                       'q004-option04': 'cold and non-fuzzy things',
                       'q004-other': 'yes',
                       'q004-likert01': 'yes',
                       'q004-likert02': 'no',
                       'q004-likert03': 'maybe so',
                       
                       'q005-type': 'textinput',
                       'q005-text': 'Enter your name:',
                       
                       'q006-type': 'textarea',
                       'q006-text': 'Enter something else!',
          
                       'q007-type': 'pagebreak',
                       
                       'q008-type': 'select',
                       'q008-text': 'Pick a color:',
                       'q008-option01': 'red',
                       'q008-option02': 'blue',
                       'q008-option03': 'green',
                       'q008-option04': 'yellow',
          
                       'q009-type': 'pagebreak',
                       
                       'q010-type': 'text', 
                       'q010-text': 'Thanks for taking this survey!'
                      }

# these are defaults not currently used.
DEFAULT_USER_ID     = 'test'
DEFAULT_PASSWORD    = 'test'

# this is meta-information about this survey software.
VERSION             = ' v. 0.17'
SURVEY_NAME         = 'EASY SURVEY'
SURVEY_BYLINE       = VERSION + ' by Morgan Ames (morganya at sims.berkeley.edu)'
SURVEY_TOOL_NAME    = SURVEY_NAME + SURVEY_BYLINE
SURVEY_TOOL_NAME_LINK = '<a href="http://www.ocf.berkeley.edu/~morganya/scripts/survey.html">'+\
                        SURVEY_NAME+'</a>'+SURVEY_BYLINE

welcome_text        = '<h2>' + SURVEY_NAME + '</h2>' + SURVEY_BYLINE + '''<br>
Easily create, edit, distribute, and output data from online surveys.<br>
(Note that this software is still being developed; if you have comments or
suggestions, please email me!)'''
