'''This package deals with the creation of surveys.
You can create a survey through a text representation
(see test_text in config.py for an example).
A web interface may be added later (TODO).
'''

import survey, auth, open_close, participants
from helpers import *

def direct(dict):
    '''Direct the form input to the right destination.
    '''
    session = auth.get_session()
    user = auth.check_session(session)
    option = dict.get(EDIT_OPTION_NAME)
    if option == INPUT_SURVEY_CGI or option == EDIT_SURVEY:  # create/edit survey
        overwrite_flag = ''
        if option == EDIT_SURVEY:
            overwrite_flag = 'yes'
        if INPUT_METHOD in dict:
            if dict[INPUT_METHOD] == TEXT_INPUT:
                edit_text(dict.get(TEXT_INPUT, ''), dict.get(TITLE, ''), overwrite=overwrite_flag)
            if dict[INPUT_METHOD] == SURVEY_ID:
                if dict.get(SURVEY_ID, make_survey_id(sample_survey_title)) == make_survey_id(sample_survey_title):
                    text = ''
                    title = ''
                else:
                    temp_survey = survey.survey(survey_id=dict[SURVEY_ID])
                    text = temp_survey.get(RAWTEXT)
                    title = temp_survey.get(TITLE)
                edit_text(text, title, overwrite=overwrite_flag)
            elif dict[INPUT_METHOD] == WEB_INPUT:
                edit_web(dict, overwrite=overwrite_flag)
    elif option == PREVIEW_SURVEY_CGI:
        if INPUT_METHOD in dict:
            if dict[INPUT_METHOD] == TEXT_INPUT:
                preview_text(dict)
            if dict[INPUT_METHOD] == SURVEY_ID:
                temp_survey = survey.survey(survey_id=dict[SURVEY_ID])
                temp_dict = {TEXT_INPUT: temp_survey.get(RAWTEXT),
                             TITLE: temp_survey.get(TITLE)}
                preview_text(temp_dict)
            elif dict[INPUT_METHOD] == WEB_INPUT:
                preview(dict)
    elif option == VIEW_RESULTS_CGI and SURVEY_ID in dict:
        survey.get_results(dict)
    elif option == SAVE_SURVEY_CGI:
        if INPUT_METHOD in dict:
            if dict[INPUT_METHOD] == TEXT_INPUT:
                save_text(dict)
            elif dict[INPUT_METHOD] == WEB_INPUT:
                save(dict)
    elif option == PROCESS_SURVEY_CGI:
        print forward(PROCESS_SURVEY_CGI+'?'+SURVEY_ID+'='+dict.get(SURVEY_ID, ''))
    elif option == TESTING:
        print forward(PROCESS_SURVEY_CGI+'?'+SURVEY_ID+'='+dict.get(SURVEY_ID, '')+'&'+TESTING+'=yes')
    elif option == OPEN_CLOSE_SURVEY and SURVEY_ID in dict:
        print open_close.form(dict[SURVEY_ID], session_cgi(INPUT_SURVEY_CGI))
    elif option == SET_OPEN_CLOSE and SURVEY_ID in dict:
        if open_close.run(dict, user):
            welcome()
        else:
            print open_close.form(dict[SURVEY_ID], session_cgi(INPUT_SURVEY_CGI))
    elif option == ENTER_PARTICIPANTS:
        if SURVEY_ID in dict:
            print participants.enter_participants_form(dict[SURVEY_ID])
        else:
            welcome("Please select a survey.")
    elif option == DELETE_SURVEY:
        if SURVEY_ID in dict:
            delete_survey(dict[SURVEY_ID])
        else:
            welcome("Select a survey to delete.")
    else:
        welcome()

def session_cgi(CGI):
    '''Add the session key to the end of the string passed in.
    '''
    return CGI + auth.get_session()

def rec_remove(path):
    try:
        for file in os.listdir(path):
            rec_remove(path+file)
        os.rmdir(path)
    except(OSError):
        # non-directory base case
        os.remove(path)

def delete_survey(surveyid):
    '''Delete the survey specified by the survey id.
    '''
    # TODO move to trash instead??
    results = []
    user = auth.check_session(auth.get_session())
    if not user:
        results.append("Please log in again.") # TODO login form
    if not isowner(user, surveyid):
        results.append("You don't own a survey by that name.")
    else:
        try:
            rec_remove(get_survey_dir(surveyid, data=0))
            auth.delete(user, SURVEYS, surveyid)
            results.append("The survey " + surveyid + " and all responses have been permanently deleted.")
        except(OSError):
            results.append("An OSError occurred during the deletion of " + surveyid + ".  The survey was probably not deleted entirely.")
    welcome('<p>'.join(results))

def move_survey(old_survey_id, new_survey_id, overwrite=0):
    '''
    '''
    results = []
    user = auth.check_session(auth.get_session())
    if not user:
        results.append("Please log in again.") # TODO login form
    elif auth.get_owner(new_survey_id) and not user == auth.get_owner(new_survey_id):
        # new survey name already exists and belongs to someone else
        results.append("The new survey name you've entered has been taken.  Please choose another.")
    if not isowner(user, old_survey_id):
        results.append("You don't own a survey by that name.")
    if isowner (user, new_survey_id) and not overwrite:
        results.append("You already have a survey by the new name you entered.")
    if not results:
        old_dir = get_survey_dir(old_survey_id, data=0)
        new_dir = get_survey_dir(new_survey_id, data=0)
        os.rename(old_dir, new_dir)
        auth.set(user, SURVEYS, new_survey_id)
        auth.delete(user, SURVEYS, old_survey_id)
    return results

def add_user(survey_id, user):
    '''
    '''
    if not survey_id == make_survey_id(sample_survey_title):
        auth.set(user, SURVEYS, survey_id)

def isowner(user, survey_id):
    return survey_id in auth.get(user, SURVEYS)

def get_survey_names(user):
    '''Return the surveys created by the specified user.
    '''
    surveys = auth.get(user, SURVEYS)
    surveys.insert(0, make_survey_id(sample_survey_title))
    return surveys

def survey_options(user):
    '''Return a string of surveys, formatted to be options
       in a "select" field.
    '''
    string = '\n    <table>\n'
    selected = ' checked'
    for name in get_survey_names(user):
        if (name == make_survey_id(sample_survey_title)) and \
            (get_survey_dir(make_survey_id(sample_survey_title)) not in listdir(SURVEY_PATH)):
            # TODO sample_survey_dir doesn't exist - create!
            pass
        temp_survey = survey.survey(survey_id=name)
        title = temp_survey.get(TITLE)
        num_results = len(temp_survey.get_result_ids())
        openness = temp_survey.get_openness()
        string += '    <tr>\n      <td valign="top">' + \
                  '<input type="radio" name="' + SURVEY_ID + \
                  '" value="' + name + '"' + selected + '></td>\n' + \
                  '      <td class="weclome_option"><b>' + title + '</b><br>' + \
                  '(' + str(num_results) + ' responses, currently ' + openness + ')' + \
                  '</td>\n    </tr>\n'
        selected = ''
    return string + '    </table>\n'

def welcome(message=''):
    '''Print the main page from which one can create, edit, preview, test,
       or take surveys.
    '''  
    session = auth.get_session()
    user = auth.check_session(session)
    if not user:
        check_login()
        return
    print get_header() + welcome_text + '\n<p>\n<div class="error">' + message + '''<p></div>
<form action="''' + session_cgi(INPUT_SURVEY_CGI) + '''" method="POST">
<input type="hidden" name="''' + INPUT_METHOD + '" value="' + SURVEY_ID + '''">
<table>
<tr>
  <td colspan="2" align="right"><a href="logout.cgi">logout</a></td>
</tr>
<tr>
  <td valign="top" class="surveys">
    <h3>Select survey:</h3>''' + survey_options(user) + '''
  </td>
  <td valign="top" class="options">
    <h3>Select option:</h3>
    <table>'''
    selected = ' checked'
    FULL_OPTIONS = EDIT_OPTIONS + [[VIEW_RESULTS_CGI, '<b>View or download results:</b>']]
    for option in FULL_OPTIONS:
        print '\n    <tr>\n      <td valign="top"><input type="radio" name="' + EDIT_OPTION_NAME + \
              '" value="' + option[0] + '"' + selected + '></td>\n      <td valign="top">' + \
              option[1] + '</td>\n    </tr>'
        selected = ''
    print '''
    <tr>
      <td valign="top"></td>
      <td valign="top">
        <b>Format:</b>
        <table>
        <tr>
          <td valign="top"><input type="radio" name="'''+FORMAT+'''" value="'''+CSV_TEXT+'''" checked></td>
          <td valign="top">CSV (comma-separated values) with text values for multiple-choice questions</td>
        </tr>
        <tr>
<!---TODO
          <td valign="top"><input type="radio" name="'''+FORMAT+'''" value="'''+CSV_NUM+'''"></td>
          <td valign="top">CSV with numeric values for multiple-choice questions</td>
        </tr>
        <tr>
          <td valign="top"><input type="radio" name="'''+FORMAT+'''" value="'''+SPSS_DATA+'''"></td>
          <td valign="top">SPSS data file</td>
        </tr>
        <tr>
          <td valign="top"><input type="radio" name="'''+FORMAT+'''" value="'''+SPSS_VARS+'''"></td>
          <td valign="top">SPSS variable file</td>
        </tr>
        <tr>
          <td valign="top"><input type="radio" name="'''+FORMAT+'''" value="'''+HTML_FORMAT+'''"></td>
          <td valign="top">HTML</td>
        </tr>
        <tr>
-------->
          <td valign="top"><input type="radio" name="'''+FORMAT+'''" value="'''+QUESTIONS_ONLY+'''"></td>
          <td valign="top">The question text in a CSV file</td>
        </tr>
        </table>
        <p>
        <b>Include:</b>
        <table>
        <tr>
          <td valign="top"><input type="radio" name="'''+INCLUDE+'''" value="'''+INCLUDE_ALL+'''" checked></td>
          <td valign="top">All questions</td>
        </tr>
        <tr>
          <td valign="top"><input type="radio" name="'''+INCLUDE+'''" value="'''+INCLUDE_MULTI+'''"></td>
          <td valign="top">Multiple-choice questions only</td>
        </tr>
        <tr>
          <td valign="top"><input type="radio" name="'''+INCLUDE+'''" value="'''+INCLUDE_TEXT+'''"></td>
          <td valign="top">Text questions only</td>
        </tr>
        </table>
      </td>
    </tr>
    </table>
    <input type="submit" value="Submit">
    </form>
  </td>
</tr>
'''
    if SHOW_ACCOUNTMGMT:
        print '''
<tr>
  <td colspan="2">
    <br>
    <h3>Account management:</h3>
      <form method="post" action="'''+session_cgi(CHANGE_USERINFO_CGI)+'''">
      <input type="radio" name="'''+DESTINATION+'" value="'+CHANGE_PASSWORD+'''">Change password<br>
      <input type="radio" name="'''+DESTINATION+'" value="'+CHANGE_EMAIL+'''">Change email from ''' + \
      auth.get(auth.check_session(auth.get_session()), EMAIL) + '''<br>
      <input type="submit" value="Submit">
      </form>
  </td>
</tr>'''
    print '</table>' + get_footer()

 
def edit_text(text, title, errors=[], overwrite=''):
    '''Display the input text for editing with "preview survey" buttons.
    '''
    if not text:
        text = test_text
    text = html_escape(text, reverse=1)
    hidden_overwrite_fields = ''
    if overwrite:
        hidden_overwrite_fields = '\n<input type="hidden" name="'+OVERWRITE+'" value="yes">\n' + \
                                  '<input type="hidden" name="' + OLD_SURVEY_ID + '" value="' + \
                                  make_survey_id(title) + '">\n'
    print get_header() + '<h3>Edit survey:</h3>\n<div class="error">' + \
          '<p>\n'.join(errors) + '''<p></div>
<table>
<tr>
  <td class="button">
    <form action="''' + session_cgi(INPUT_SURVEY_CGI) + '''" method="POST">
    <!--<input type="submit" value="Cancel">-->
    </form>
  </td>
  <td class="button">
    <form action="''' + session_cgi(INPUT_SURVEY_CGI) + '''" method="POST">
    <input type="hidden" name="''' + EDIT_OPTION_NAME + '" value="' + PREVIEW_SURVEY_CGI + '">' + \
    hidden_overwrite_fields + '''
    <!--<input type="submit" value="Preview">-->
  </td>
  <td></td>
</tr>
<tr>
  <td colspan="3">
<input type="hidden" name="''' + INPUT_METHOD + '" value="' + TEXT_INPUT + '''">

Survey title: <input type="text" name="''' + TITLE + '" value="' + title + \
'''" size="30"> (required)

<textarea name="''' + TEXT_INPUT + '" rows="' + str(TEXT_INPUT_ROWS) + \
'" cols="' + str(TEXT_INPUT_COLS) + '">' + text + '''</textarea>
  </td>
</tr>
<tr>
  <td class="button">
    <input type="submit" value="Preview">
    </form>
  </td>
  <td class="button">
    <form action="''' + session_cgi(INPUT_SURVEY_CGI) + '''" method="POST">
    <input type="submit" value="Cancel">
    </form>
  </td>
</tr>
<tr>
  <td colspan="3">''' + survey_formatting + '''
  </td>
</tr>
</table>
''' + get_footer()

def preview_text(dict):
    '''Parse the input text and display resulting survey with
       "change" and "save" buttons 
    '''
    errors = []
    text = dict.get(TEXT_INPUT, '')
    if TITLE not in dict:
        errors.append("Please enter a title.")
    if errors:
        edit_text(text, dict.get(TITLE, ''), errors)
    else:
        survey_dict = parse_text(html_escape(text, 1), dict[TITLE])
        for field in SAVED_FIELDS: # includes overwrite
            if field in dict:
                survey_dict[field] = dict[field]
        preview(survey_dict)

def save_text(dict):
    title = dict.get(TITLE, '')
    text  = dict.get(TEXT_INPUT, '')
    survey_dict = parse_text(html_escape(text, 1), title)
    for field in SAVED_FIELDS:
        if field in dict:
            survey_dict[field] = dict[field]
    save(survey_dict)

def parse_text_errors(text):
    errors = []
    if not text:
        errors.append('No text was entered.  Please go back and enter survey text.')
    # TODO more
    return errors

def parse_text(text, title):
    '''Return a dictionary of survey questions given well-formatted text input.
The output dictionary is one that can be read by the survey class
initialization function.

Following is a sample input to modify for your own survey.

    ''' + test_text
    errors = parse_text_errors(text)
    if errors:
        edit_text(text, title, errors)
        sys.exit()
    else:
        dict = {}
        # split into questions
        text = text.replace('\r', '')
        questions = list_strip(text.split('\n\n'))
        # iterate through questions
        q_index = 0
        for question in questions:
            q_index += 1
            dict.update(question_parse(question, get_code(q_index)))
        dict[TITLE] = title
        dict[INPUT_METHOD] = TEXT_INPUT
        dict[RAWTEXT] = text
        return dict

def question_parse(question, code):
    '''Parse the question and return a dictionary containing it.
    '''
    dict = {}
    # vertical radio
    if question.count('\n   *'):
        dict[get_key(code, QTYPE)] = V_RADIO
        lines = list_strip(question.split('\n   *'))
        dict[get_key(code, QTEXT)] = htmlize(lines.pop(0))
        dict[get_key(code, QOPTIONS)] = lines
    # vertical checkbox
    elif question.count('\n   #'):
        dict[get_key(code, QTYPE)] = V_CHECKBOX
        lines = list_strip(question.split('\n   #'))
        dict[get_key(code, QTEXT)] = htmlize(lines.pop(0))
        dict[get_key(code, QOPTIONS)] = lines
    # drop-down box
    elif question.count('\n   -'):
        dict[get_key(code, QTYPE)] = SELECT
        lines = list_strip(question.split('\n   -'))
        dict[get_key(code, QTEXT)] = htmlize(lines.pop(0))
        dict[get_key(code, QOPTIONS)] = lines
    # horizontal radio
    elif question.count('\t*'):
        dict[get_key(code, QTYPE)] = H_RADIO
        (question_text, the_rest) = list_strip(question.split('\n\t', 1))
        dict[get_key(code, QTEXT)] = htmlize(question_text)
        lines = list_strip(the_rest.split('\n'))
        dict[get_key(code, QLIKERT)] = list_strip(lines.pop(0).strip().split('\t'))
        for i in range(len(lines)):
            lines[i] = lines[i].replace('\t*', '')
        dict[get_key(code, QOPTIONS)] = lines
    # horizontal checkbox        
    elif question.count('\t#'):
        dict[get_key(code, QTYPE)] = H_CHECKBOX
        (question_text, the_rest) = list_strip(question.split('\n\t', 1))
        dict[get_key(code, QTEXT)] = htmlize(question_text)
        lines = list_strip(the_rest.split('\n'))
        dict[get_key(code, QLIKERT)] = list_strip(lines.pop(0).strip().split('\t'))
        for i in range(len(lines)):
            lines[i] = lines[i].replace('\t#', '')
        dict[get_key(code, QOPTIONS)] = lines
    # textarea
    elif question.count('|---') and \
         question.count('---|'):
        dict[get_key(code, QTYPE)] = TEXTAREA
        i = question.index('|---')
        dict[get_key(code, QTEXT)] = htmlize(question[:i].strip())
        lines = question[i:].split('\n')
        dict[get_key(code, QWIDTH)] = len(lines.pop(0).strip('|'))
        dict[get_key(code, QHEIGHT)] = len(lines)
    # text box
    elif question.count('___'):
        dict[get_key(code, QTYPE)] = TEXTINPUT
        i = question.index('___')
        dict[get_key(code, QTEXT)] = htmlize(question[:i].strip())
        dict[get_key(code, QWIDTH)] = question[i:].count('_')
    # password box 
    elif question.count('***'):
        dict[get_key(code, QTYPE)] = PASSWORD
        i = question.index('***')
        dict[get_key(code, QTEXT)] = htmlize(question[:i].strip())
        dict[get_key(code, QWIDTH)] = question[i:].count('*')
    # reset button
    elif question.strip().lower() == '[reset]':
        dict[get_key(code, QTYPE)] = RESET
    # pagebreak 
    elif question.upper().count('--PAGEBREAK--'):
        dict[get_key(code, QTYPE)] = PAGEBREAK
    # title - deprecated (title now entered separately)
    # elif question.strip().lower().startswith('[title]'):
    #    dict[TITLE] = htmlize(question.strip()[7:])
    #    q_index -= 1
    # page title
    elif question.strip().lower().startswith('[pagetitle]'):
        dict[get_key(code, QTYPE)] = PAGETITLE
        dict[get_key(code, QTEXT)] = htmlize(question.strip()[11:])
    else: # default is text
        dict[get_key(code, QTYPE)] = TEXT
        dict[get_key(code, QTEXT)] = htmlize(question.strip())
    return dict

def question_help(question_name):
    return get_header() + \
      '<table width="'+str(int(HELP_WIDTH)-30)+'"><tr><td>\n' + \
      '<b>Question text:</b>\n</td></tr>\n' + \
      '<tr><td bgcolor="#eeeeee">\n' + \
      '<pre width="'+str(int(HELP_WIDTH)/7)+'">' + \
      question_preview(question_name, rawtext=1) + \
      '</pre>\n<br><br></td></tr>\n' + \
      '<tr><td><b>Formatted question:</b></td></tr>\n' + \
      '<tr><td bgcolor="#eeeeee">\n' + \
      question_preview(question_name) + \
      '</td></tr></table>\n'

def question_preview(question_name='', question_dict={}, rawtext=0):
    '''Return the HTML of a single question for preview purposes.
    '''
    if rawtext and question_name:
        return question_examples.get(question_name, '')
    to_return = ''
    if not question_dict:
        for question in question_examples.get(question_name, '').replace('\r', '').split('\n\n'):
            question_dict = question_parse(question, '')
            to_return += survey.question(question_dict).__str__()
    return to_return

def make_survey_id(string, temp=0):
    if string:
        id = form_escape(string)
        if not id.startswith('_'):
            id = '_' + id
    else:
        id = '_survey01' # TODO error!
    return id

def edit_web(dict, errors=[], overwrite=''):
    pass # TODO may be coming in a later version, if needed

def parse_web_errors(dict):
    return [] # TODO error-handling for web input, if/when implemented

def hide_web_input(dict):
    # TODO either put entire contents of survey in hidden fields or
    # save it temporarily somewhere and return a pointer
    # (must change edit for this).
    # Make sure INPUT_METHOD=WEB_INPUT is in the dict or add it.
    return ''

def hide_text_input(dict):
    to_return = '<input type="hidden" name="' + INPUT_METHOD + \
                '" value="' + TEXT_INPUT + '">\n' + \
                '<input type="hidden" name="' + TEXT_INPUT + \
                '" value="' + html_escape(dict[RAWTEXT]) + '">\n'
    for field in SAVED_FIELDS:
        if field in dict:
            to_return += '<input type="hidden" name="' + field + \
                         '" value="' + dict[field] + '">\n'
    return to_return

def preview(dict, errors=[]):
    if dict[INPUT_METHOD] == WEB_INPUT:
        if parse_web_errors(dict):
            return # TODO error!
        form_contents = hide_web_input(dict)
    elif dict[INPUT_METHOD] == TEXT_INPUT:
        form_contents = hide_text_input(dict)
    if SURVEY_ID not in dict:
        dict[SURVEY_ID] = make_survey_id(dict.get(TITLE, ''), temp=1)
    temp_survey = survey.survey(dict)
    header = get_header()
    footer = get_footer()
    print header + '<h3>Preview survey:</h3>\n<div class="error">' + \
          '<p>\n'.join(errors) + '''<p></div>
Below is a preview of your survey.
Note that all pages of the survey are previewed on one page.
Horizontal lines represent page breaks.
<p>
<table>
<tr>
  <td class="button">  
    <form action="''' + session_cgi(INPUT_SURVEY_CGI) + '''" method="POST">
    <input type="hidden" name="''' + EDIT_OPTION_NAME + '" value="' + SAVE_SURVEY_CGI + '''">
    ''' + form_contents + '''
    <input type="submit" value="Save">
    </form>
  </td>
  <td class="button">
    <form action="''' + session_cgi(INPUT_SURVEY_CGI) + '''" method="POST">
    <input type="hidden" name="''' + EDIT_OPTION_NAME + '" value="' + INPUT_SURVEY_CGI + '''">
    ''' + form_contents + '''
    <input type="submit" value="Edit">
    </form>
  </td>
  <td class="button">
    <form action="''' + session_cgi(INPUT_SURVEY_CGI) + '''" method="POST">
    <input type="submit" value="Cancel">
    </form>
  </td>
  <td></td>
</tr>
<tr>
  <td colspan="4">
    <hr>

''' + temp_survey.__str__().lstrip('Content-Type: text/html').rstrip(footer) + '''

  </td>
</tr>
<tr>
  <td class="button">
    <form action="''' + session_cgi(INPUT_SURVEY_CGI) + '''" method="POST">
    <input type="hidden" name="''' + EDIT_OPTION_NAME + '" value="' + SAVE_SURVEY_CGI + '''">
    ''' + form_contents + '''
    <input type="submit" value="Save">
    </form>
  </td>
  <td class="button">
    <form action="''' + session_cgi(INPUT_SURVEY_CGI) + '''" method="POST">
    <input type="hidden" name="''' + EDIT_OPTION_NAME + '" value="' + INPUT_SURVEY_CGI + '''">
    ''' + form_contents + '''
    <input type="submit" value="Edit">
    </form>
  </td>
  <td class="button">
    <form action="''' + session_cgi(INPUT_SURVEY_CGI) + '''" method="POST">
    <input type="submit" value="Cancel">
    </form>
  </td>
  <td></td>
</tr>
</table>

''' + footer

def check_login(dict={}, dest=DEFAULT_DEST):
    form_dict = {DESTINATION: dest}
    if dict.get(INPUT_METHOD, '') == WEB_INPUT:
        form_dict[HIDDEN] = hide_web_input(dict) # TODO fix if/when adding webinput
    elif dict.get(INPUT_METHOD, '') == TEXT_INPUT:
        form_dict[INPUT_METHOD] = TEXT_INPUT
        form_dict[TEXT_INPUT] = html_escape(dict[RAWTEXT])
    auth.print_forms(auth.login_form(form_dict) + \
                     auth.newuser_form(form_dict) + \
                     auth.forgot_form(form_dict))

def save(dict):
    user = auth.check_session(auth.get_session())
    if not user:
        check_login(dict, SAVE_SURVEY_CGI)
    else:
        errors = []
        if TITLE not in dict:
            errors.append('You must enter a survey title. ')
        else:
            temp_survey_id = dict.get(SURVEY_ID, make_survey_id(dict[TITLE]))
            owner = auth.get_owner(temp_survey_id)
            if owner:
                if owner == user:
                    if not dict.get(OVERWRITE, ''):
                        dict[OVERWRITE] = 'yes'
                        preview(dict, ['Do you want to overwrite your previously-saved survey ' + \
                                       'with this name? If yes, click "save" again. ' + \
                                       ' Otherwise, click "edit" and change the name.'])
                else:
                    errors.append('That survey title has been taken by another user. ' + \
                                  ' Please enter another.')
            if temp_survey_id == make_survey_id(sample_survey_title):
                errors.append('Edits to the sample survey will not be saved.  If you want to save the survey, please change its name.')
        if errors:
            if dict[INPUT_METHOD] == TEXT_INPUT:
                edit_text(dict[RAWTEXT], dict[TITLE], errors)
            else:
                edit_web(dict, errors)
        else:
            new_id = dict.get(SURVEY_ID, make_survey_id(dict[TITLE]))
            old_id = dict.get(OLD_SURVEY_ID, new_id)
            dict[SURVEY_ID] = new_id
            temp_survey = survey.survey(dict)
            if dict.get(OVERWRITE, '') and \
               not old_id == new_id and \
               not old_id == make_survey_id(sample_survey_title):
                # overwrite is set and the survey has been renamed: need to move survey
                # (but if it's the sample survey, just copy it, don't rename)
                errors = move_survey(old_id, new_id)
                if errors:
                    if dict[INPUT_METHOD] == TEXT_INPUT:
                        edit_text(dict[RAWTEXT], dict[TITLE], errors)
                    else:
                        edit_web(dict, errors)
            if not errors:
                temp_survey.save(user)
                add_user(dict[SURVEY_ID], user)
                url = get_survey_url(new_id)
                print get_header() + '''
Your survey has been saved.  It can be accessed from the following URL:
<blockquote><a href="''' + url + '">' + url + '''</a>
</blockquote>
<a href="'''+session_cgi(INPUT_SURVEY_CGI)+'''">Return to editing index</a>
<hr>
''' + get_footer()
